local n_chunk = 16

local n_chunk_total = n_chunk * n_chunk * 4

script.on_event(defines.events.on_player_created, function(event)
    local player = game.players[event.player_index]
    player.force.chart(player.surface, {{-200, -200}, {200, 200}})
    player.print("Server will be closed when map is generated!")
end)

script.on_event(defines.events.on_tick, function(event)
    if game.tick == 2 then
        print"# request the chunk generation"
        local surface = game.surfaces[1]
        surface.request_to_generate_chunks({0, 0}, n_chunk)
        surface.always_day = true
        game.speed = 1000
    end
end)

local n_chunk_generated = 0
local last_percent = 0

local function take_map_shot(options)
    local left   = options.area[1][1]
    local top    = options.area[1][2]
    local right  = options.area[2][1]
    local bottom = options.area[2][2]

    local width = right - left
    local height = bottom - top

    local surface = game.surfaces[1]

    local data = {string.format("P6 %d %d 255\n", width, height)}

    for y = options.area[1][2], options.area[2][2] - 1 do
        for x = options.area[1][1], options.area[2][1] - 1 do
            local tile = surface.get_tile(x, y)
            if tile and tile.valid then
                local color = tile.prototype.map_color
                table.insert(data, string.char(color.r, color.g, color.b))
            else
                table.insert(data, string.char(127, 127, 127))
            end
        end
    end

    local color = {
        coal = string.char(1, 1, 1),
        ["crude-oil"] = string.char(
            math.floor(0.8 * 255),
            math.floor(0.1 * 255),
            math.floor(0.8 * 255)
        ),
        ["copper-ore"] = string.char(
            math.floor(0.803 * 255),
            math.floor(0.388 * 255),
            math.floor(0.215 * 255)
        ),
        ["iron-ore"] = string.char(
            math.floor(0.337 * 255),
            math.floor(0.419 * 255),
            math.floor(0.427 * 255)
        ),
        ["stone"] = string.char(
            math.floor(0.478 * 255),
            math.floor(0.450 * 255),
            math.floor(0.317 * 255)
        ),
    }

    if options.with_resources then
        for _, r in ipairs(surface.find_entities_filtered{
            area = options.area,
            type = "resource"
        }) do
            if color[r.name] then
                local x = math.floor(r.position.x)
                local y = math.floor(r.position.y)

                if r.name == 'crude-oil' then
                    for dx = -1, 1 do
                        for dy = -1, 1 do
                            if left <= x + dx and x + dx < right
                                and top <= y + dy and y + dy < right
                            then
                                local idx = 2
                                    + (y + dx - options.area[1][2]) * width
                                    + (x + dy - options.area[1][1])
                                data[idx] = color[r.name]
                            end
                        end
                    end
                elseif options.full_resources or (math.floor(x/2) + math.floor(y/2)) % 2 == 1 then
                    local idx = 2
                        + (y - options.area[1][2]) * width
                        + (x - options.area[1][1])

                    data[idx] = color[r.name]
                end
            end
        end -- foreach entity
    end -- with_resources

    local binary = table.concat(data, '')
    game.write_file(options.filename, binary, false)
end

local gen_requested = false

script.on_event(defines.events.on_chunk_generated, function(event)
    -- local player = game.players[1]
    local surface = game.surfaces[1]

    local left = event.area.left_top.x
    local top = event.area.left_top.y
    local right = event.area.right_bottom.x
    local bottom = event.area.right_bottom.y

    if n_chunk * -32 <= left and left < n_chunk * 32 
        and n_chunk * -32 <= top and top < n_chunk * 32
    then
        n_chunk_generated = n_chunk_generated + 1
        if n_chunk_generated == n_chunk_total then
            print("# all chunks generated")
            game.speed = 1
        else
            local percent = math.floor(n_chunk_generated / n_chunk_total * 100 / 5) * 5
            if (percent ~= last_percent) then
                print(string.format("# %d of %d chunks generated", n_chunk_generated, n_chunk_total))
                last_percent = percent
            end
            return
        end
    else
        -- ignore chunks outside our requested view
        return
    end

    local map_gen = surface.map_gen_settings
    local filename = string.format("seed_%d-terrain_%s-water_%s.ppm",
            map_gen.seed, map_gen.terrain_segmentation, map_gen.water
            )

    take_map_shot{
        area = {{n_chunk * -32, n_chunk * -32}, {n_chunk * 32, n_chunk * 32}},
        filename = filename,
        with_resources = true
    }

    print("Screenshot: " .. filename)
end)
